using System;
using System.Messaging;
using System.Security;
using System.Security.Permissions;
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection ;

namespace FileSecurityCS
{

	public class FileSystemAccessControlEntry: AccessControlEntry
	{
		public FileSystemAccessControlEntry():base()
		{
		}

		public FileSystemAccessControlEntry(Trustee id):base(id)
		{
		}

		public FileSystemAccessControlEntry(Trustee id, GenericAccessRights rights, StandardAccessRights stdrights, AccessControlEntryType acetype):base(id, rights, stdrights, acetype)
		{
		}

		public FileAccessRights FileAccessRights
		{
			get
			{
				return ((FileAccessRights)CustomAccessRights);
			}
			set 
			{
				CustomAccessRights = (int)value;
			}
		}
	}


	public enum FileAccessRights 
	{
		FileRead = 0x1,
		FileWrite = 0x2,
		FileAppend = 0x4,
		FileReadProperties = 0x8,
		FileWriteProperties = 0x10,
		FileExecute = 0x20,
		FileReadAttributes = 0x80,
		FileWriteAttributes = 0x100,
		DirectoryList = 0x1,
		DirectoryAddFile = 0x2,
		DirectoryAddSubdirectory = 0x4,
		DirectoryDeleteChild = 0x40,
		DirectoryTraverse = 0x20
	}

	public enum FileSystemInheritanceType 
	{
		Default = 0,
		AllowInheritance = 1,
		BlockInheritance = 2
	}


	public class FileSystemAccessControlList: AccessControlList
	{

		private const int MAXIMUM_ALLOWED  = 0x2000000;
		private const int GENERIC_ALL  = 0x10000000;
		private const int SE_FILE_OBJECT  = 1;
		private const uint DACL_SECURITY_INFORMATION  = 0x4;
		private const uint PROTECTED_DACL_SECURITY_INFORMATION = 0x80000000;
		private const uint UNPROTECTED_DACL_SECURITY_INFORMATION  = 0x20000000;

		[DllImport("advapi32.dll", SetLastError=true, CharSet=CharSet.Auto  )]extern static private int GetNamedSecurityInfo  (String filename, int ObjectType, uint SecurityInfo,  int ppsidOwner,  int ppsidGroup, out int ppDacl, int  ppSacl,  int ppSecurityDescriptor);
		[DllImport("advapi32.dll", SetLastError=true, CharSet=CharSet.Auto  )]extern static private int SetNamedSecurityInfo (String filename, int ObjectType, uint SecurityInfo, int psidOwner, int psidGroup, int pDacl, int pSacl);


		private FileSystemInheritanceType m_InheritanceType;
		bool m_CombineExisting ;

		public FileSystemInheritanceType InheritanceSettings
		{
			get
			{
				return(m_InheritanceType);
			}
			set
			{
				m_InheritanceType = value;
			}
		}

		public bool CombineExisting
		{
			get
			{
				return(m_CombineExisting);
			}
			set
			{
				m_CombineExisting = value;
			}
		}

		public void SetFileSecurity(string FileOrDirectory)
		{
			// SecurityInfo is the flag that specifies what type of security settings to set/retrieve
			// Default is DACL, but we add the inheritance setting if a non-default is requested
			uint SecurityInfo  = DACL_SECURITY_INFORMATION;
			switch(m_InheritanceType) 
			{
				case FileSystemInheritanceType.AllowInheritance:
					SecurityInfo = SecurityInfo | UNPROTECTED_DACL_SECURITY_INFORMATION;
					break;
				case FileSystemInheritanceType.BlockInheritance:
					SecurityInfo = SecurityInfo | PROTECTED_DACL_SECURITY_INFORMATION;
					break;
			}
		
			FileInfo fi = new FileInfo(FileOrDirectory);
			string fullpath;
			// First see if it is a file that exists


			if (! fi.Exists)
			{
				// On failure, check if it's a directory.
				DirectoryInfo di = new DirectoryInfo(FileOrDirectory);
				if (! di.Exists ) throw new FileNotFoundException(FileOrDirectory);
				fullpath = di.FullName;
			}
			else 
			{
				fullpath = fi.FullName;
			}

			// Demand the appropriate file access permission
			FileIOPermission fileperm = new FileIOPermission(FileIOPermissionAccess.AllAccess, fullpath);
			fileperm.Demand();

			// If allowed Then assert unmanaged code permission and reflection permission
			SecurityPermission useperm1 = new SecurityPermission(SecurityPermissionFlag.UnmanagedCode);
			ReflectionPermission useperm2 = new ReflectionPermission(ReflectionPermissionFlag.MemberAccess);
			PermissionSet usepermlist = new PermissionSet(PermissionState.None);
			usepermlist.AddPermission(useperm1);
			usepermlist.AddPermission(useperm2);
			usepermlist.Assert();

			try 
			{
				int siresult;
				int oldACL = 0;
				// If we're combining existing ACL entries, retrieve the existing ACL entries
				if (m_CombineExisting)
				{
					siresult = GetNamedSecurityInfo(fullpath, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION,  0, 0, out oldACL, 0, 0);
					if (siresult != 0)
					{
						// If it fails, throw the appropriate Win32 exception
						throw new System.ComponentModel.Win32Exception(Marshal.GetLastWin32Error());
					}
				}
				int newACL;

				// Now call the private MakeAcl method to get a new ACL that optionally merges the existing one.
				IntPtr newActlPtr = (IntPtr)(base.GetType().BaseType.InvokeMember("MakeAcl", BindingFlags.InvokeMethod | BindingFlags.Instance | BindingFlags.NonPublic, null, this, new Object[] {new IntPtr(oldACL)}));
				newACL = newActlPtr.ToInt32();


				// And finally, set the new ACL into the file object
				siresult = SetNamedSecurityInfo(fullpath, SE_FILE_OBJECT, SecurityInfo, 0, 0, newACL, 0);

				// Free the ACL
				this.GetType().BaseType.InvokeMember("FreeAcl", BindingFlags.InvokeMethod | BindingFlags.Static | BindingFlags.NonPublic, null, this, new Object[] {new IntPtr(newACL)});

				if (siresult != 0)
				{
					throw new System.ComponentModel.Win32Exception(Marshal.GetLastWin32Error());
				}
			}
			catch (Exception ex)
			{
				throw(ex);
			}
		}
	}


}
